' Program: QGXDEMO.BAS (QBASIC/QuickBASIC Demo of G12864)
' This program demonstrates the G12864 Serial Graphics LCD
' from Scott Edwards Electronics Inc. with QBASIC or QuickBASIC.
' Configure the G12864 for 9600 baud (config switch 2 ON) and
' escape codes (config switch 4 OFF). Connect power and serial
' input. This program uses COM1--change the "OPEN" instruction if
' you're using COM2.

'===============Delay Subroutine using Nulls====================
DECLARE SUB pause (time AS INTEGER)
DEFINT A-Z

'===========Constants Defining LCD Instructions=================
' These constants assign names to the LCD instructions. The
' numbers are the ASCII values listed in the manual. For example,
' cntl-L is the clear-screen instruction; the ASCII value is 12.
' So printing CHR$(12) to the LCD clears the screen.
CONST clrTXT = 12       ' Cntl-L: clear the text layer.
CONST bksp = 8          ' Backspace.
CONST posCmd = 16       ' Position the cursor.
CONST ESC = 27          ' Escape code.

'==================Demonstration Program=======================
' Set up serial port COM1; disable handshaking (CD, CS, DS, OP).
OPEN "COM1:9600,N,8,1,CD0,CS0,DS0,OP0" FOR OUTPUT AS #1
' ===========
' ==Recipe 1
' ===========
' This code works the same as recipe 1 from the G12864 manual.
' It clears the screen, prints some text, then positions the
' cursor to change just portions of the text. Notice that some
' PRINT statements end in semicolons (;) and others don't. PRINT
' normally adds a carriage return to all data. When you don't want
' a return (as when you send instructions), adding a semicolon
' prevents it.
PRINT #1, CHR$(clrTXT);         ' Clear text layer.
PRINT #1, CHR$(ESC); "C0.";     ' Clear graphics layer.
PRINT #1, "RECIPE NUMBER 1"     ' Print the title, followed by return.
PRINT #1, "Power:"  ' Print label, followed by return.
PRINT #1, "Speed:"  ' "     "
' Here's the cursor-positioning command. This PRINT statement combines
' the command with the text to be printed. Notice that there's a
' throwaway character (.) separating the number (23) from the text "75".
' You could use any character except null (ASCII 0) in that position.
PRINT #1, CHR$(posCmd); "23.75%";       ' Go to position 23, print "75%"
PRINT #1, CHR$(posCmd); "39.87mph";     ' Go to position 39, print "87mph"
SLEEP 2         ' Delay for 2 seconds.
' Here is the other way to send position values--as single bytes with
' no throwaway. Just add 64 to the position, and send that as a CHR$(),
' which is just BASIC's term for a byte. The G12864 will accept _any_
' number in this shorthand, so it's a valuable technique to learn.
PRINT #1, CHR$(posCmd); CHR$(64 + 23); "80%";         ' Pos 23, print "80%"
PRINT #1, CHR$(posCmd); CHR$(64 + 39); "93mph";       ' Pos 39, print "93mph"
' Here's a looped version with variables. Note that when BASIC formats a
' variable as text--as when you PRINT a variable like power--it adds spaces
' before and after. If you don't like the spaces, convert the value to
' a string with STR$() and use LTRIM$() and/or RTRIM$() to trim spaces.
' See the BASIC docs for more info.
pwrPos = 64 + 23        ' Position of the power data.
spdPos = 64 + 39        ' Position of the speed data.
FOR power = 50 TO 85 STEP 5
  SLEEP 1
  PRINT #1, CHR$(posCmd); CHR$(pwrPos); power; "%";
  speed = power * 1.2   ' Phoney calculation.
  PRINT #1, CHR$(posCmd); CHR$(spdPos); speed; "mph"
NEXT
' ===========
' ==Recipe 2
' ===========
' Recipe 2 shows how to display EEPROM screens.
PRINT #1, CHR$(clrTXT);         ' Clear text layer.
PRINT #1, CHR$(ESC); "E2.";     ' Display EPROM image #2.
SLEEP 2 ' Delay 2 seconds.
PRINT #1, CHR$(ESC); "C0.";     ' Clear graphics layer.
PRINT #1, CHR$(ESC); "E6.";     ' Display EPROM image #6.
FOR flashes = 1 TO 3
  SLEEP 1
  PRINT #1, CHR$(ESC); "R1.";   ' Reverse graphics.
  SLEEP 1
  PRINT #1, CHR$(ESC); "R0.";   ' Normal graphics.
NEXT
SLEEP 2 ' Pause 2 seconds.
' ===========
' ==Recipe 3
' ===========
' Recipe 3 shows a useful technique for combining text and graphics to
' selectively highlight portions of the text.
PRINT #1, CHR$(clrTXT);         ' Clear text layer.
PRINT #1, CHR$(ESC); "C0.";     ' Clear graphics layer.
PRINT #1, " Menu Item 1"        ' Display text, followed by return.
PRINT #1, " Menu Item 2"        ' "
PRINT #1, " Menu Item 3"        ' "
PRINT #1, " Menu Item 4"        ' "
PRINT #1, CHR$(ESC); "O1.";     ' Set (O)verlay to 1 (XOR).
FOR selection = 8 TO 11         ' EEPROM screens 8-11 are highlight bars.
  SLEEP 1           ' Pause 1 second.
  hilite = selection + 64       ' One-byte shortcut for selection.
  PRINT #1, CHR$(ESC); "E"; CHR$(hilite);       ' Show selection screen.
NEXT
SLEEP 2 ' Pause 2 seconds.
' ===========
' ==Recipe 4
' ===========
' This code draws a box on the screen with the line and line-to
' commands. It's a little different from the recipe in the manual
' in that it uses only shortcut numbers instead of numeric text.
PRINT #1, CHR$(clrTXT);         ' Clear text layer.
PRINT #1, CHR$(ESC); "C0.";     ' Clear graphics layer.
x1 = 10 + 64: y1 = 10 + 64      ' 1st point (64 added to make shortcuts)
x2 = 117 + 64: y2 = 10 + 64     ' 2nd point.
' Here's the line command in the form <esc> L x1 y1 x2 y2
PRINT #1, CHR$(ESC); "L"; CHR$(x1); CHR$(y1);
PRINT #1, CHR$(x2); CHR$(y2);
' Now we can use line-to commands. This draws a line that starts
' from the endpoint of the previous line.
y2 = 53 + 64        ' New y coordinate
PRINT #1, CHR$(ESC); "T"; CHR$(x2); CHR$(y2);
x2 = 10 + 64        ' New x coordinate
PRINT #1, CHR$(ESC); "T"; CHR$(x2); CHR$(y2);
y2 = 10 + 64        ' New y coordinate
PRINT #1, CHR$(ESC); "T"; CHR$(x2); CHR$(y2);
SLEEP 2
' ===========
' ==Recipe 5
' ===========
' This code demonstrates the new features of firmware version 060 and
' higher, included in the surface-mount version 2.0 hardware or
' available as an upgrade to older units. A small (6x8-pixel) font
' image must be installed in flash EEPROM screen 15 for this to
' work properly.
PRINT #1, CHR$(clrTXT);         ' Clear text layer.
PRINT #1, CHR$(ESC); "C0.";     ' Clear graphics layer.
PRINT #1, "= New Features =";   ' Print banner in default font.
PRINT #1, CHR$(ESC); "F1.";     ' Switch to small font.
PRINT #1, "The G12864 now has"  ' Print messages in small font.
PRINT #1, " multiple fonts, "
PRINT #1, " highlight bars,  "
PRINT #1, " faster line-draw,"
PRINT #1, " quick ESC-Z reset,"
PRINT #1, " and SMT hardware.";
' Now we'll selectively highlight lines with ESC H. This instruction
' erases the graphics layer, draws 8-pixel-high bar(s) on it, and
' sets the XOR relationship between text and graphics layers.
SLEEP 2
PRINT #1, CHR$(ESC); "H3.";     ' Highlight line 1 of big font.
SLEEP 1
PRINT #1, CHR$(ESC); "H8.";     ' Highlight line 3 of small font.
SLEEP 1
PRINT #1, CHR$(ESC); "H16.";    ' Highlight line 4 of small font.
SLEEP 1
PRINT #1, CHR$(ESC); "H32.";    ' Highlight line 5 of small font.
SLEEP 1
PRINT #1, CHR$(ESC); "H64.";    ' Highlight line 6 of small font.
SLEEP 1
PRINT #1, CHR$(ESC); "H128.";   ' Highlight line 7 of small font.
SLEEP 2
PRINT #1, CHR$(ESC); "Z3.";     ' Reset text and graphics settings.
' ===========
' ==Bonus
' ===========
' This code shows graphs a sine function using the plot command.
' Since sine is a value from -1 to +1, it multiplies this value by 25
' to get +/- 25, and adds 31 to center it vertically on the screen.
' It uses shortcut values of x and y to fill in the <esc> P x y
' command.
PRINT #1, CHR$(clrTXT);         ' Clear text layer.
PRINT #1, "sine waves";         ' Print label.
PRINT #1, CHR$(ESC); "C0.";     ' Clear graphics layer.
FOR x = 0 TO 127    ' Plot across entire screen.
  y = 31 + INT(25 * (SIN((3.14156 / 20) * x)))  ' Calculate y.
  PRINT #1, CHR$(ESC); "P"; CHR$((x) + 64); CHR$((y) + 64);     ' Plot xy.
NEXT
END

